//----------------------------------------------------------------------------
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "timer.h"
//----------------------------------------------------------------------------
// prescaler config
// 1:1, 2:8, 3:32, 4:64,  5:128,  6:256,    7:1024   - ATmega128
// 1:1, 2:8, 3:64, 4:256, 5:1024, 6:ExtT0L, 7:ExtT0H - ATmega32 / ATmega644 / ATmega328
#define TIMER_PRESCALER_CFG   3
#define TIMER_PRESCALER       64
#define TIMER_OCR0            ((F_CPU / TIMER_PRESCALER / (1000 / TIMER_INTERVAL_MS)) - 1)
#if (TIMER_OCR0 == 0 || TIMER_OCR0 > 255)
  #error "Bad value of macro TIMER_OCR0"
#endif
//----------------------------------------------------------------------------
static volatile uint32_t total_ticks;
//----------------------------------------------------------------------------
void timer_init(void)
{
  // set-up TIMER 0
  TCCR0A = _BV(WGM01);
  TCCR0B = TIMER_PRESCALER_CFG;
  OCR0A = TIMER_OCR0;
  TIMSK0 |= _BV(OCIE0A);

  total_ticks = 0;
}
//----------------------------------------------------------------------------
uint32_t timer_get_ticks(void)
{
  return total_ticks;
}
//----------------------------------------------------------------------------
void timer_wait_msecs(uint16_t msecs)
{
  uint32_t end_ticks;

  if (msecs == 0)
    return;

  end_ticks = total_ticks + msecs;

  do {
    sleep_mode();
  } while (total_ticks < end_ticks);
}
//----------------------------------------------------------------------------
void timer_wait_secs(uint8_t secs)
{
  uint32_t end_ticks;

  if (secs == 0)
    return;

  end_ticks = total_ticks + (secs * (1000 / TIMER_INTERVAL_MS));

  do {
    sleep_mode();
  } while (total_ticks < end_ticks);
}
//----------------------------------------------------------------------------
ISR(TIMER0_COMPA_vect)
{
  total_ticks++;
}
//----------------------------------------------------------------------------
